@@ -1,11 +1,9 @@
-# (c) Sound Object Logic 2000-2001
-
-use strict;
-
 package Tangram::Cursor;
 
-use vars qw( $stored %done );
+use strict;
+use Tangram::Cursor::Data;
 use Carp;
+use vars qw( $stored %done );
 
 sub new
 {
@@ -68,16 +66,21 @@ sub select
 	$self->{-desc} = $args{desc};
 	$self->{-distinct} = $args{distinct};
 	$self->{-limit} = $args{limit};
+	$self->{-noexec} = $args{noexec};
 
 	# with outer queries, each remote object is either inside or
 	# outside the query.
 	my ($inner_objects, $outer_objects)
 	    = (Set::Object->new(), Set::Object->new());
 
+	#kill 2, $$;
 	if (exists $args{retrieve}) {
 	    $self->retrieve( @{ $args{retrieve} } );
-	    # assume that objects are outside the query until joined
-	    $outer_objects->insert
+	    # assume that objects are inside the query until joined.
+	    my $which = ($self->{TARGET}
+			 ? $outer_objects
+			 : $inner_objects);
+	    $which->insert
 		( map { $_->{objects}->members }
 		  @{ $args{retrieve} } );
 	}
@@ -86,7 +89,7 @@ sub select
 	my (@filter_from, @filter_where);
 	$inner_objects->insert($target->object) if $target;
 
-	my $filter = Tangram::Filter->new( tight => 100,
+	my $filter = Tangram::Expr::Filter->new( tight => 100,
 					   objects => $inner_objects );
 	my ($seen_inner, $outer);
 
@@ -101,20 +104,30 @@ sub select
 
 	# anything mentioned in the `outer_filter' is part of the
 	# outer query
+	#kill 2, $$;
+	my $is_outer;
+	if ( my $forced_outer = $args{force_outer} ) {
+	    my @outer = ( map { $_->object }
+			  ( ref $forced_outer eq "ARRAY"
+			    ? @$forced_outer
+			    : $forced_outer));
+	    $is_outer = 1;
+	    $inner_objects->remove(@outer);
+	    $outer_objects->insert(@outer);
+	    $filter->{objects}->remove(@outer);
+	}
 	if (my $outer_filter = $args{outer_filter}) {
 	    #kill 2, $$;
-	    if ( my $forced_outer = $args{force_outer} ) {
-		$inner_objects->remove(map { $_->object }
-				       @$forced_outer);
-	    }
 
-	    $outer = Tangram::Filter->new( tight => 100,
+	    $outer = Tangram::Expr::Filter->new( tight => 100,
 					   objects => $outer_objects );
 	    $outer->{expr} = $outer_filter->{expr};
 	    $outer->{objects}->insert($outer_filter->{objects}->members);
 	    $outer->{objects}->remove($inner_objects->members);
+	    $is_outer = 1;
 
-	} elsif ( $outer_objects->size ) {
+	}
+	if ( !$is_outer and $outer_objects->size ) {
 
 	    # If there is no outer query, then we must add the
 	    # selected tables to the inner query part.
@@ -144,7 +157,10 @@ sub select
 					( $outer
 					  ? ([ $outer->from ],
 					     [ $outer->where ],
-					    ) : () )
+					    ) : () ),
+					($args{force_outer}
+					 ? (any_outer => 1)
+					 : () )
 				      ),
 		   undef, $_ ]
 	     } @polysel
@@ -152,7 +168,7 @@ sub select
 
 	$self->{position} = -1;
 
-	return $self->execute();
+	return $self->execute() unless delete $self->{-noexec};
 }
 
 sub execute
@@ -163,6 +179,30 @@ sub execute
 	$self->prepare_next_statement() && $self->next();
   }
 
+# XXX - not reached by test suite
+sub sql_string
+  {
+      my $self = shift;
+
+      if ( $self->{_last_sql} ) {
+	  print STDERR "RETURNING FROM _last_sql\n";
+	  return $self->{_last_sql};
+      }
+      elsif ( $self->{ACTIVE} ) {
+	  print STDERR "RETURNING FROM ACTIVE\n";
+	  return $self->{ACTIVE}[0];
+      }
+      elsif ( $self->{cur_select} and @{$self->{cur_select}} ) {
+	  print STDERR "RETURNING FROM CUR_SELECT\n";
+	  return $self->{cur_select}[0][0];
+      }
+      elsif ( $self->{SELECTS} ) {
+	  print STDERR "RETURNING FROM SELECTS\n";
+	  return $self->{SELECTS}[0][0];
+      }
+
+  }
+
 sub prepare_next_statement
   {
 	my ($self) = @_;
@@ -182,6 +222,7 @@ sub prepare_next_statement
 
 	$self->{sth} = $sth;
 
+
 	$sth->execute() or croak "Execute failed; $DBI::errstr";
 
 	return $sth;
@@ -189,23 +230,28 @@ sub prepare_next_statement
 
 sub build_select
 {
-	my ($self, $template, $cols, $from, $where, $ofrom, $owhere)
-	    = @_;
+	my ($self, $template, $cols, $from, $where, $ofrom, $owhere,
+	    @options) = @_;
 
 	if (my $retrieve = $self->{-retrieve})
 	{
 		@$cols = map { $_->{expr} } @$retrieve;
 	}
 
+	# this needs a hack to get right...
+	if ( $self->{-limit} ) {
+	    @options = $self->{STORAGE}->limit_sql($self->{-limit});
+	}
+
 	my $select = $template->instantiate
 	    ( $self->{TARGET}, $cols, $from, $where,
 	      ( $self->{-group} ? (group => $self->{-group}) : () ),
 	      ( $self->{-order} ? (order => $self->{-order}) : () ),
 	      ( $self->{-distinct} ? (distinct => $self->{-distinct}) : () ),
-	      ( $self->{-limit} ? (limit => $self->{-limit}) : () ),
 	      ( $self->{-desc} ? (desc => $self->{-desc}) : () ),
 	      ( $ofrom ? ( ofrom => $ofrom ) : () ),
 	      ( $owhere ? ( owhere => $owhere ) : () ),
+	      @options,
 	    );
 	
 
@@ -240,12 +286,25 @@ sub _next
 
 	    my $class = $storage->{id2class}{$classId} or die "unknown class id $classId";
 
-	    # even if object is already loaded we must read it so that @rpw only contains residue
-	    my $obj = $storage->read_object($id, $class, $state);
+	    my $obj = $storage->{objects}{$id};
+
+	    # even if object is already loaded we must read it so that
+	    # @row only contains residue, with -retrieve, and so that
+	    # any foreign/intrusive collections can get the
+	    # information they need.
+	    if ( !defined($obj) or $self->{-retrieve} or
+		 $self->{-no_skip_read} ) {
+		print $Tangram::TRACE __PACKAGE__.": reading object $id\n"
+		    if $Tangram::TRACE and $Tangram::DEBUG_LEVEL > 0;
+		$obj = $storage->read_object($id, $class, $state);
+	    } else {
+		print $Tangram::TRACE __PACKAGE__.": not reading object $id\n"
+		    if $Tangram::TRACE and $Tangram::DEBUG_LEVEL > 0;
+		# just pretend we did it.
+		@row = ();
+	    }
 
 	    # if object is already loaded return previous copy
-	    $obj = $storage->{objects}{$id} if exists $storage->{objects}{$id};
-
 	    $self->{-current} = $obj;
 
 	} else {
@@ -297,68 +356,11 @@ sub residue
 	@{shift->{-residue}};
 }
 
+# XXX - not reached by test suite
 sub object
 {
 	my ($self) = @_;
 	return $self->{_object};
 }
 
-package Tangram::DataCursor;
-
-use Carp;
-
-sub open
-{
-	my ($type, $storage, $select, $conn) = @_;
-   
-	confess unless $conn;
-	
-	bless
-	{
-	 select => $select,
-	 storage => $storage,
-	 cursor => $storage->sql_cursor(substr($select->{expr}, 1, -1), $conn),
-	}, $type;
-}
-
-sub fetchrow
-{
-	my $self = shift;
-	my @row = $self->{cursor}->fetchrow;
-	return () unless @row;
-	map { $_->{type}->read_data(\@row) } @{$self->{select}{cols}};
-}
-
-sub fetchall_arrayref
-{
-	my $self = shift;
-	my @results;
-
-	while (my @row = $self->fetchrow)
-	{
-		push @results, [ @row ];
-	}
-
-	return \@results;
-}
-
-sub new
-{
-	my $pkg = shift;
-	return bless [ @_ ] , $pkg;
-}
-
-sub DESTROY
-  {
-	my $self = shift;
-	$self->close();
-  }
-
-sub close
-  {
-	my $self = shift;
-	$self->{cursor}{connection}->disconnect()
-	  unless $self->{cursor}{connection} == $self->{storage}{db};
-  }
-
 1;
