@@ -8,6 +8,7 @@ sub new
 	bless [ @_ ], $pkg;
 }
 
+# XXX - not tested by test suite
 sub exists
 {
 	my ($self, $expr, $filter) = @_;
@@ -23,7 +24,7 @@ sub exists
 			);
 	}
 
-	my $expr_str = $expr->{expr};
+	my $expr_str = $expr->expr;
 	$expr_str =~ tr/\n/ /;
 
 	return Tangram::Expr::Filter->new( expr => "exists $expr_str", tight => 100,
