@@ -1,204 +1,11 @@
 
 
-use strict;
-
-package Tangram::ClassHash;
-
-use Carp;
-
-sub class
-{
-   my ($self, $class) = @_;
-   $self->{$class} or croak "unknown class '$class'";
-}
-
-package Tangram::Node;
-
-sub get_bases
-  {
-	@{ shift->{BASES} }
-  }
-
-*direct_bases = \&get_bases;
-
-sub get_specs
-  {
-	@{ shift->{SPECS} }
-  }
-
-sub for_conforming
-{
-   my ($class, $fun, @args) = @_;
-   my $done = Set::Object->new;
-
-   my $traverse;
-
-   $traverse = sub {
-	 my $class = shift;
-	 return if $done->includes($class);
-	 $done->insert($class);
-	 $fun->($class, @args);
-
-	 foreach my $derived (@{ $class->{SPECS} }) {
-	   $traverse->($derived);
-	 }
-   };
-
-   $traverse->($class);
- }
-
-#---------------------------------------------------------------------
-#  Tangram::Node->for_composing($closure, @_)
-#
-# Runs the given closure once for this class, and all its superclasses
-# listed in the schema as $class->{BASES}
-#
-#---------------------------------------------------------------------
-sub for_composing
-{
-   my ($class, $fun, @args) = @_;
-   my $done = Set::Object->new;
-
-   my $traverse;
-
-   $traverse = sub {
-	 my $class = shift;
-	 return if $done->includes($class);
-	 $done->insert($class);
-
-	 foreach my $base (@{ $class->{BASES} }) {
-	   $traverse->($base);
-	 }
-
-	 $fun->($class, @args);
-   };
-
-   $traverse->($class);
- }
-
-sub get_exporter {
-  my ($self, $context) = @_;
-
-  return $self->{EXPORTER} ||= do {
-
-	my (@export_sources, @export_closures);
-
-	$self->for_composing
-	    ( sub {
-		  my ($part) = @_;
-
-		  $context->{class} = $part;
-
-		  for my $field ($part->direct_fields()) {
-		      if (my $exporter = $field->get_exporter($context)) {
-			  if (ref $exporter) {
-			      push @export_closures, $exporter;
-			      push @export_sources, 'shift(@closures)->($obj, $context)';
-			  } else {
-			      push @export_sources, $exporter;
-			  }
-		      }
-		  }
-	      } );
-
-	my $export_source = join ",\n", @export_sources;
-	my $copy_closures =
-	    ( @export_closures ? ' my @closures = @export_closures;' : '' );
-
-	# $Tangram::TRACE = \*STDOUT;
-
-	$export_source = ("sub { my (\$obj, \$context) = \@_;"
-			  ."$copy_closures\n$export_source }");
-
-	print $Tangram::TRACE "Compiling exporter for $self->{name}...\n".($Tangram::DEBUG_LEVEL > 1 ? "$export_source\n" : "")
-	    if $Tangram::TRACE;
-
-	eval $export_source or die;
-    }
-}
-
-sub get_importer {
-  my ($self, $context) = @_;
-
-  return $self->{IMPORTER} ||= do {
-	my (@import_sources, @import_closures);
-
-	$self->for_composing
-	    ( sub {
-		  my ($part) = @_;
-
-		  $context->{class} = $part;
-
-		  for my $field ($part->get_direct_fields()) {
-
-		      my $importer = $field->get_importer($context)
-			  or next;
-
-		      if (ref $importer) {
-			  push @import_closures, $importer;
-			  push @import_sources, 'shift(@closures)->($obj, $row, $context)';
-		      } else {
-			  push @import_sources, $importer;
-		      }
-		  }
-	      } );
-
-	my $import_source = join ";\n", @import_sources;
-	my $copy_closures =
-	    ( @import_closures ? ' my @closures = @import_closures;' : '' );
-
-	# $Tangram::TRACE = \*STDOUT;
-
-	$import_source = ( "sub { my (\$obj, \$row, \$context) = \@_;"
-			   ."(ref(\$row) eq 'ARRAY') and (\@\$row) or Carp::confess('no row!');\n"
-			   ."# line 1 'tangram-$self->{table}-to-$self->{name}.pl'\n"
-			   ."$copy_closures\n$import_source }" );
-
-	print $Tangram::TRACE "Compiling importer for $self->{name}...\n".($Tangram::DEBUG_LEVEL > 1 ? "$import_source\n" : "")."\n"
-	  if $Tangram::TRACE;
-
-	# use Data::Dumper; print Dumper \@cols;
-	eval $import_source or die;
-  };
-}
-
-package Tangram::Class;
-use vars qw(@ISA);
- @ISA = qw( Tangram::Node );
-
-sub members
-{
-   my ($self, $type) = @_;
-   return @{$self->{$type}};
-}
-
-sub is_root
-  {
-	!@{ shift->{BASES} }
-  }
-
-sub get_direct_fields
-  {
-	map { values %$_ } values %{ shift->{fields} }
-  }
-
-sub get_table { shift->{table} }
-
-*direct_fields = \&get_direct_fields;
-
-sub get_import_cols {
-  my ($self, $context) = @_;
-  my $table = $self->{table};
-  map { map { [ $table, $_ ] } $_->get_import_cols($context) } $self->get_direct_fields()
-}
-
-sub get_export_cols {
-  my ($self, $context) = @_;
-  my $table = $self->{table};
-  map { map { [ $table, $_ ] } $_->get_export_cols($context) } $self->get_direct_fields()
-}
-
 package Tangram::Schema;
+
+use Tangram::Schema::ClassHash;
+use Tangram::Schema::Class;
+
+use strict;
 #our @ISA = qw( SelfLoader );
 use Carp;
 
@@ -207,10 +14,11 @@ use vars qw( %TYPES );
 %TYPES = 
 (
    %TYPES,
-#   ref      => new Tangram::Ref,
+#   ref      => new Tangram::Type::Ref::FromMany,
 );
 
 use Scalar::Util qw(reftype weaken);
+use Tangram::Util qw(pretty);
 
 sub new
 {
@@ -249,7 +57,7 @@ sub new
     my @class_list = reftype($self->{'classes'}) eq 'HASH' ? %{ $self->{'classes'} } : @{ $self->{'classes'} };
     my $class_hash = $self->{'classes'} = {};
 
-    bless $class_hash, 'Tangram::ClassHash';
+    bless $class_hash, 'Tangram::Schema::ClassHash';
 
     my $autoid = 0;
 
@@ -264,7 +72,7 @@ sub new
 		  $classdef->{id} = ++$autoid;
 		}
 
-		bless $classdef, 'Tangram::Class';
+		bless $classdef, 'Tangram::Schema::Class';
 
 		$classdef->{name} = $class;
 		$classdef->{table} ||= $self->{normalize}->($class, 'tablename');
@@ -292,10 +100,16 @@ sub new
 			    foreach (keys %$fields);
 		    }
 		}
+		die "'fields' must be a hash ref ($classdef->{name}), but is "
+		    .pretty($classdef->{fields})
+		    if defined $classdef->{fields} and
+			reftype $classdef->{fields} ne "HASH";
+
 		$classdef->{members} = $classdef->{fields};
 
 		my $cols = 0;
 
+
 		foreach my $typetag (keys %{$classdef->{members}})
 		{
 			my $memdefs = $classdef->{members}{$typetag};
@@ -378,23 +192,26 @@ sub check_class
 sub classdef
 {
    my ($self, $class) = @_;
-   return $self->{classes}{$class} or confess "unknown class '$class'";
+   return $self->{classes}{$class} || confess "unknown class '$class'";
 }
 
 *get_class_by_name = \&classdef;
 
+# XXX - not tested by test suite
 sub classes
 {
    my ($self) = @_;
    return keys %{$self->{'classes'}};
 }
 
+# XXX - not tested by test suite
 sub direct_members
 {
    my ($self, $class) = @_;
    return $self->{'classes'}{$class}{member_type};
 }
 
+# XXX - not tested by test suite
 sub all_members
 {
    my ($self, $class) = @_;
@@ -410,6 +227,7 @@ sub all_members
 	$members;
 }
 
+# XXX - not tested by test suite
 sub all_bases
 {
    my ($self, $class) = @_;
@@ -417,6 +235,7 @@ sub all_bases
 	$self->visit_down($class, sub { @{ $classes->{shift()}{bases} } } );
 }
 
+# XXX - not tested by test suite
 sub find_member
 {
    my ($self, $class, $member) = @_;
@@ -459,6 +278,7 @@ sub find_member_class
    $result;
 }
 
+# XXX - not tested by test suite
 sub visit_up
 {
    my ($self, $class, $fun) = @_;
@@ -507,6 +327,7 @@ sub _visit_down
    @results
 }
 
+# XXX - not tested by test suite
 sub for_bases
 {
    my ($self, $class, $fun) = @_;
@@ -559,6 +380,7 @@ sub _for_each_spec
 
 }
 
+# XXX - not tested by test suite
 sub declare_classes
 {
    my ($self, $root) = @_;
@@ -577,6 +399,7 @@ sub declare_classes
    }
 }
 
+# XXX - not tested by test suite
 sub is_persistent
 {
    my ($self, $x) = @_;
