@@ -30,7 +30,12 @@ eval "use $dialect";
 ($dialect = 'Tangram::Relational'), eval("use $dialect") if $@;
 print $Tangram::TRACE "Vendor driver $dialect not found - using ANSI SQL ($@)\n"
     if $@ and $Tangram::TRACE;
-print $Tangram::TRACE "Using dialect $dialect\n" if $Tangram::TRACE;
+if ($Tangram::TRACE) {
+    print $Tangram::TRACE "DBConfig.pm: dialect = $dialect, cparm = $cs, "
+	.($user ? "$user" : "(no user)").", "
+	    .($passwd ? ("x" x (length $passwd)) : "(no passwd)")."\n";
+}
+
 
 our $AUTOLOAD;
 sub AUTOLOAD {
@@ -43,4 +48,18 @@ sub cparm {
     return ($cs, $user, $passwd);
 }
 
+sub setup {
+    my $class = shift if UNIVERSAL::isa($_[0], __PACKAGE__);
+    $class ||= __PACKAGE__;
+    my $schema = shift;
+    my ($dsn, $u, $p) = $class->cparm;
+    my $dbh = DBI->connect($dsn, $u, $p);
+    eval {
+	local $dbh->{RaiseError};
+	local $dbh->{PrintError};
+	Tangram::Relational->retreat($schema, $dbh);
+    };
+    Tangram::Relational->deploy($schema, $dbh);
+}
+
 1;
